using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI;
using System.Configuration;
using System.Data.SqlClient;
using System.Web.UI.WebControls;

namespace RussellMason.Data.SqlClient
{

	/// <summary>
	/// Provides page state persitance to SQL Server.
	/// </summary>
	public class SqlPageStatePersister : PageStatePersister
	{

		#region Fields

		private const string _hiddenFieldID = "__VIEWSTATEKEY"; 

		#endregion Fields

		#region Constructors

		/// <summary>
		/// Initializes a new instance of the SqlPageStatePersister class.
		/// </summary>
		/// <param name="page">
		/// The page that provides the state to persist.
		/// </param>
		public SqlPageStatePersister(Page page) : base(page)
		{
		} 

		#endregion Constructors

		#region Properties

		/// <summary>
		/// Gets the connection string to the database that provides stored state.
		/// </summary>
		public string ConnectionString
		{
			get
			{
				string connectionString = null;

				ConnectionStringSettings settings =
					ConfigurationManager.ConnectionStrings["SqlPageStatePersisterConnectionString"];

				if (settings != null)
				{
					connectionString = settings.ConnectionString;
				}

				return connectionString;
			}
		} 

		#endregion Properties

		#region Methods

		/// <summary>
		/// Loads state from SQL Server.
		/// </summary>
		public override void Load()
		{
			string key = this.Page.Request.Form[_hiddenFieldID];

			if (!string.IsNullOrEmpty(key))
			{
				using (SqlConnection connection = new SqlConnection(this.ConnectionString))
				{
					SqlCommand command = new SqlCommand("GetWebClientPageState", connection);
					command.CommandType = System.Data.CommandType.StoredProcedure;

					command.Parameters.Add("@StateKey", System.Data.SqlDbType.UniqueIdentifier);
					command.Parameters["@StateKey"].Value = new Guid(key);

					connection.Open();

					object rawState = command.ExecuteScalar();

					connection.Close();

					if ((rawState != null) && (rawState != DBNull.Value))
					{
						string stateText = Convert.ToString(rawState);

						Pair statePair = this.StateFormatter.Deserialize(stateText) as Pair;

						if (statePair != null)
						{
							this.ViewState = statePair.First;
							this.ControlState = statePair.Second;
						}
					}
				}
			}
		}

		/// <summary>
		/// Saves state to SQL Server.
		/// </summary>
		public override void Save()
		{
			if ((this.ViewState != null) || (this.ControlState != null))
			{
				Guid stateKey = Guid.NewGuid();
				string key = this.Page.Request.Form[_hiddenFieldID];

				if (!string.IsNullOrEmpty(key))
				{
					stateKey = new Guid(key);
				}
				
				Pair statePair = new Pair(this.ViewState, this.ControlState);

				string stateText = this.StateFormatter.Serialize(statePair);

				using (SqlConnection connection = new SqlConnection(this.ConnectionString))
				{
					SqlCommand command = new SqlCommand("SetWebClientPageState", connection);
					command.CommandType = System.Data.CommandType.StoredProcedure;

					command.Parameters.Add("@StateKey", System.Data.SqlDbType.UniqueIdentifier);
					command.Parameters["@StateKey"].Value = stateKey;

					command.Parameters.Add("@State", System.Data.SqlDbType.Text);
					command.Parameters["@State"].Value = stateText;

					connection.Open();

					command.ExecuteNonQuery();

					connection.Close();
				}

				this.Page.ClientScript.RegisterHiddenField(_hiddenFieldID, stateKey.ToString());
			}
		} 

		#endregion Methods

	}

}
