using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI;
using System.Data.SqlClient;
using System.Web.UI.WebControls;
using System.IO;
using System.Web.UI.HtmlControls;

namespace RussellMason.Web.Application.Data.SqlClient
{

	/// <summary>
	/// Provider for saving and loading page state to SQL Server.
	/// </summary>
	public class SqlPageStateProvider : IPageStateProvider
	{

		#region Fields

		private const string _hiddenFieldID = "__SQLVIEWSTATE";

		private string _connectionString = null;

		#endregion Fields

		#region Constructors

		/// <summary>
		/// Initializes a new instance of the SqlPageStateProvider class.
		/// </summary>
		/// <param name="connectionString">
		/// The database connection string.
		/// </param>
		public SqlPageStateProvider(string connectionString)
		{
			if (connectionString == null)
			{
				throw new ArgumentNullException("connectionString");
			}

			_connectionString = connectionString;
		} 

		#endregion Constructors

		#region Properties

		/// <summary>
		/// Gets the connection string to the database that provides stored state.
		/// </summary>
		public string ConnectionString
		{
			get
			{
				return _connectionString;
			}
		} 

		#endregion Properties

		#region IViewStateProvider Members

		/// <summary>
		/// Loads the page state from SQL Server and returns the state object.
		/// </summary>
		/// <param name="page">
		/// The page to load state into.
		/// </param>
		/// <returns>
		/// The page state.
		/// </returns>
		public object Load(Page page)
		{
			object state = null;
			string key = page.Request.Form[_hiddenFieldID];

			if (!string.IsNullOrEmpty(key))
			{
				using (SqlConnection connection = new SqlConnection(this.ConnectionString))
				{
					SqlCommand command = new SqlCommand("GetWebClientPageState", connection);
					command.CommandType = System.Data.CommandType.StoredProcedure;

					command.Parameters.Add("@StateKey", System.Data.SqlDbType.UniqueIdentifier);
					command.Parameters["@StateKey"].Value = new Guid(key);

					connection.Open();

					object rawState = command.ExecuteScalar();

					if ((rawState != null) && (rawState != DBNull.Value))
					{
						string stateText = Convert.ToString(rawState);

						LosFormatter losFormatter = new LosFormatter();
						state = losFormatter.Deserialize(stateText);
					}

					connection.Close();
				}
			}

			return state;
		}

		/// <summary>
		/// Saves the specified state to SQL Server.
		/// </summary>
		/// <param name="page">
		/// The page to save the state of.
		/// </param>
		/// <param name="state">
		/// The state to stave.
		/// </param>
		public void Save(Page page, object state)
		{
			StringWriter stringWriter = new StringWriter();
			LosFormatter losFormatter = new LosFormatter();
			losFormatter.Serialize(stringWriter, state);
			string stateText = stringWriter.ToString();

			Guid stateKey = Guid.NewGuid();
			string key = page.Request.Form[_hiddenFieldID];

			if (!string.IsNullOrEmpty(key))
			{
				stateKey = new Guid(key);
			}

			page.ClientScript.RegisterHiddenField(_hiddenFieldID, stateKey.ToString());

			using (SqlConnection connection = new SqlConnection(this.ConnectionString))
			{
				SqlCommand command = new SqlCommand("SetWebClientPageState", connection);
				command.CommandType = System.Data.CommandType.StoredProcedure;

				command.Parameters.Add("@StateKey", System.Data.SqlDbType.UniqueIdentifier);
				command.Parameters["@StateKey"].Value = stateKey;

				command.Parameters.Add("@State", System.Data.SqlDbType.Text);
				command.Parameters["@State"].Value = stateText;

				connection.Open();

				command.ExecuteNonQuery();

				connection.Close();
			}
		}

		#endregion IViewStateProvider Members

	}

}
