using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI.HtmlControls;
using System.ComponentModel;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Drawing;
using System.Xml;

using RussellMason.Web.UI.Office2007.Design;

namespace RussellMason.Web.UI.Office2007
{

	/// <summary>
	/// Represents a large button for use on the Ribbon Bar.
	/// </summary>
	[
	Designer(typeof(LargeRibbonButtonDesigner)),
	ToolboxData("<{0}:LargeRibbonButton runat=\"server\"></{0}:LargeRibbonButton>")
	]
	public class LargeRibbonButton : TextRibbonButton
	{

		#region Fields

		private DropDownMenuAlignment _dropDownMenuAlignment = DropDownMenuAlignment.Center;

		#endregion Fields

		#region Constructors

		/// <summary>
		/// Initializes a new instance of the LargeRibbonButton class.
		/// </summary>
		public LargeRibbonButton() : this(string.Empty)
		{
			// Shared State
			// 0 - Enabled
			// 1 - Checked
			// 2 - EnabledImageUrl
			// 3 - DisabledImageUrl
			// 4 - Text
			// 5 - DropDownMenuID
		}

		/// <summary>
		/// Initializes a new instance of the LargeRibbonButton class.
		/// </summary>
		/// <param name="id">
		/// The ID to assign to the control.
		/// </param>
		public LargeRibbonButton(string id) : this(id, string.Empty)
		{
		}

		/// <summary>
		/// Initializes a new instance of the LargeRibbonButton class.
		/// </summary>
		/// <param name="id">
		/// The ID to assign to the control.
		/// </param>
		/// <param name="text">
		/// The button text.
		/// </param>
		public LargeRibbonButton(string id, string text) : this(id, text, string.Empty, string.Empty)
		{
		}

		/// <summary>
		/// Initializes a new instance of the LargeRibbonButton class.
		/// </summary>
		/// <param name="id">
		/// The ID to assign to the control.
		/// </param>
		/// <param name="text">
		/// The button text.
		/// </param>
		/// <param name="enabledImageUrl">
		/// The URL to an image to display when enabled.
		/// </param>
		/// <param name="disabledImageUrl">
		/// The URL to an image to display when disabled.
		/// </param>
		public LargeRibbonButton(string id, string text, string enabledImageUrl, string disabledImageUrl)
		{
			this.ID = id;
			this.Text = text;
			this.EnabledImageUrl = enabledImageUrl;
			this.DisabledImageUrl = disabledImageUrl;

			// Default Width;
			this.Width = new Unit("45px");
		}

		#endregion Constructors

		#region Properties

		/// <summary>
		/// Gets or sets where the drop down arrow is positioned.
		/// </summary>
		[
		DefaultValue(typeof(DropDownMenuAlignment), "Center"),
		Description("Specifies where the drop down arrow is positioned when a drop down menu is assigned.")
		]
		public DropDownMenuAlignment DropDownMenuAlignment
		{
			get
			{
				return _dropDownMenuAlignment;
			}
			set
			{
				_dropDownMenuAlignment = value;
			}
		}  

		#endregion Properties

		#region Implementation Methods

		/// <summary>
		/// Creates the control's layout.
		/// </summary>
		protected override void CreateChildControls()
		{
			ClearControls();

			this.Attributes.Add("class", "largeRibbonButton");
			this.Attributes.Add("title", this.ToolTip);

			HtmlGenericControl startDiv = new HtmlGenericControl("div");
			startDiv.EnableViewState = false;
			startDiv.ID = "Start";
			startDiv.Attributes.Add("class", "largeRibbonButtonStart");

			this.Controls.Add(startDiv);

			HtmlGenericControl itemDiv = new HtmlGenericControl("div");
			itemDiv.EnableViewState = false;
			itemDiv.ID = "Item";
			itemDiv.Attributes.Add("class", "largeRibbonButtonItem");

			this.Controls.Add(itemDiv);

			HtmlGenericControl imageDiv = new HtmlGenericControl("div");
			imageDiv.EnableViewState = false;
			imageDiv.Attributes.Add("class", "largeRibbonButtonImage");

			itemDiv.Controls.Add(imageDiv);

			HtmlImage image = new HtmlImage();
			image.EnableViewState = false;
			image.ID = "Image";

			imageDiv.Controls.Add(image);

			HtmlGenericControl textDiv = new HtmlGenericControl("div");
			textDiv.EnableViewState = false;
			textDiv.ID = "Text";
			textDiv.Attributes.Add("class", "largeRibbonButtonText");

			itemDiv.Controls.Add(textDiv);

			HtmlGenericControl dropDownDiv = new HtmlGenericControl("div");
			dropDownDiv.EnableViewState = false;
			dropDownDiv.ID = "DropDownMenuImage";

			itemDiv.Controls.Add(dropDownDiv);

			HtmlGenericControl endDiv = new HtmlGenericControl("div");
			endDiv.EnableViewState = false;
			endDiv.ID = "End";
			endDiv.Attributes.Add("class", "largeRibbonButtonEnd");

			this.Controls.Add(endDiv);
		}

		/// <summary>
		/// Sets the internal width of the button.
		/// </summary>
		protected override void SetWidth()
		{
			int innerWidth = 0;

			if (this.Width.Value > 4)
			{
				innerWidth = Convert.ToInt32(this.Width.Value) - 4;
			}

			HtmlGenericControl itemDiv = FindControl("Item") as HtmlGenericControl;

			itemDiv.Style.Add("width", innerWidth.ToString() + "px");
		}

		/// <summary>
		/// Sets the button's checked state.
		/// </summary>
		protected override void SetCheckedState()
		{
			HtmlGenericControl startDiv = FindControl("Start") as HtmlGenericControl;
			HtmlGenericControl itemDiv = FindControl("Item") as HtmlGenericControl;
			HtmlGenericControl endDiv = FindControl("End") as HtmlGenericControl;

			if (this.Checked)
			{
				startDiv.Attributes.Add("class", "largeRibbonButtonStartChecked");
				itemDiv.Attributes.Add("class", "largeRibbonButtonItemChecked");
				endDiv.Attributes.Add("class", "largeRibbonButtonEndChecked");
			}
			else
			{
				startDiv.Attributes.Add("class", "largeRibbonButtonStart");
				itemDiv.Attributes.Add("class", "largeRibbonButtonItem");
				endDiv.Attributes.Add("class", "largeRibbonButtonEnd");
			}
		}

		/// <summary>
		/// Sets the button's text.
		/// </summary>
		protected override void SetText()
		{
			HtmlGenericControl div = FindControl("Text") as HtmlGenericControl;

			if (div != null)
			{
				if (this.Enabled)
				{
					div.Attributes.Add("class", "largeRibbonButtonText");
				}
				else
				{
					div.Attributes.Add("class", "largeRibbonButtonTextDisabled");
				}

				div.InnerText = this.Text;
			}
		}

		/// <summary>
		/// Sets whether the drop down image is visible.
		/// </summary>
		protected override void SetDropDownMenu()
		{
			HtmlGenericControl dropDownDiv = 
				FindControl("DropDownMenuImage") as HtmlGenericControl;

			if (dropDownDiv != null)
			{
				if (this.Enabled)
				{
					dropDownDiv.Attributes.Add("class", "largeRibbonButtonDropDownMenuArrow");
				}
				else
				{
					dropDownDiv.Attributes.Add("class", "largeRibbonButtonDropDownMenuArrowDisabled");
				}

				string style = null;

				switch (this.DropDownMenuAlignment)
				{
					case DropDownMenuAlignment.Left:
						style = "margin-left: 4px; margin-right: auto;";
						break;

					case DropDownMenuAlignment.Center:
						style = "margin-left: auto; margin-right: auto;";
						break;

					case DropDownMenuAlignment.Right:
						style = "margin-left: auto; margin-right: 4px;";
						break;
				}

				dropDownDiv.Attributes.Add("style", style);

				dropDownDiv.Visible = (!string.IsNullOrEmpty(this.DropDownMenuID));
			}
		}

		/// <summary>
		/// Writes the object's details to the specified writer.
		/// </summary>
		/// <param name="writer">
		/// The writer to write to.
		/// </param>
		protected override void SerializeContent(XmlWriter writer)
		{
			base.SerializeContent(writer);

			writer.WriteAttributeString("DropDownMenuAlignment", this.DropDownMenuAlignment.ToString());
		}

		/// <summary>
		/// Deserializes the object from the specified node.
		/// </summary>
		/// <param name="element">
		/// The element to get property values from.
		/// </param>
		public override void Deserialize(XmlElement element)
		{
			base.Deserialize(element);

			if (element.HasAttribute("DropDownMenuAlignment"))
			{
				string dropDownMenuAlignmentText = element.Attributes["DropDownMenuAlignment"].Value;

				DropDownMenuAlignment dropDownMenuAlignment =
					(DropDownMenuAlignment) Enum.Parse(typeof(DropDownMenuAlignment), dropDownMenuAlignmentText);

				this.DropDownMenuAlignment = dropDownMenuAlignment;
			}
		}

		#endregion Implementation Methods

	}

}
