using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;

using RussellMason.Web.UI.Office2007.Design;

namespace RussellMason.Web.UI.Office2007
{

	/// <summary>
	/// Represents a large button for use on the Ribbon Bar which is made up of 2 parts,
	/// a button and a drop down list.
	/// </summary>
	[
	Designer(typeof(LargeRibbonButtonDesigner)),
	ToolboxData("<{0}:LargeSplitRibbonButton runat=\"server\"></{0}:LargeSplitRibbonButton>")
	]
	public class LargeSplitRibbonButton : LargeRibbonButton
	{

		#region Static Properties

		/// <summary>
		/// Gets the BottomClick event key.
		/// </summary>
		public static readonly object BottomClickEvent = new object(); 

		#endregion Static Properties

		#region Fields

		private string _bottomScript = string.Empty; 

		#endregion Fields

		#region Constructors

		/// <summary>
		/// Initializes a new instance of the LargeSplitRibbonButton class.
		/// </summary>
		public LargeSplitRibbonButton() : this(string.Empty)
		{
			// Shared State
			// 0 - Enabled
			// 1 - Checked
			// 2 - EnabledImageUrl
			// 3 - DisabledImageUrl
			// 4 - Text
			// 5 - DropDownMenuID
		}

		/// <summary>
		/// Initializes a new instance of the LargeSplitRibbonButton class.
		/// </summary>
		/// <param name="id">
		/// The ID to assign to the control.
		/// </param>
		public LargeSplitRibbonButton(string id) : this(id, string.Empty)
		{
		}

		/// <summary>
		/// Initializes a new instance of the LargeSplitRibbonButton class.
		/// </summary>
		/// <param name="id">
		/// The ID to assign to the control.
		/// </param>
		/// <param name="text">
		/// The button text.
		/// </param>
		public LargeSplitRibbonButton(string id, string text) : this(id, text, string.Empty, string.Empty)
		{
		}

		/// <summary>
		/// Initializes a new instance of the LargeSplitRibbonButton class.
		/// </summary>
		/// <param name="id">
		/// The ID to assign to the control.
		/// </param>
		/// <param name="text">
		/// The button text.
		/// </param>
		/// <param name="enabledImageUrl">
		/// The URL to an image to display when enabled.
		/// </param>
		/// <param name="disabledImageUrl">
		/// The URL to an image to display when disabled.
		/// </param>
		public LargeSplitRibbonButton(string id, string text, string enabledImageUrl, string disabledImageUrl)
		{
			this.ID = id;
			this.Text = text;
			this.EnabledImageUrl = enabledImageUrl;
			this.DisabledImageUrl = disabledImageUrl;

			// Default Width;
			this.Width = new Unit("45px");
		}

		#endregion Constructors

		#region Properties

		/// <summary>
		/// Gets or sets whether the button is in a checked state or not.
		/// This value cannot be changed and must be false. Setting to true
		/// causes an exception.
		/// </summary>
		[
		DefaultValue(false),
		Description("Specifies whether the button is in a checked state or not. Must be false for this control.")
		]
		public override bool Checked
		{
			get
			{
				bool isChecked = Convert.ToBoolean(this.SharedState["Checked"].Value);

				return isChecked;
			}
			set
			{
				if (value)
				{
					string message = "Checked cannot be set to true for this control.";

					throw new InvalidOperationException(message);
				}

				this.SharedState["Checked"].Value = value;
			}
		}

		/// <summary>
		/// Gets or set the client code to execute when the bottom part of the button is clicked.
		/// </summary>
		[
		DefaultValue(""),
		Description("Specifies the client code to execute when the bottom part of the button is clicked.")
		]
		public virtual string OnClientBottomClick
		{
			get
			{
				return _bottomScript;
			}
			set
			{
				_bottomScript = value;
			}
		} 

		#endregion Properties

		#region Methods

		/// <summary>
		/// Raises the click event;
		/// </summary>
		/// <param name="eventArgument">
		/// The event data.
		/// </param>
		public override void RaisePostBackEvent(string eventArgument)
		{
			if (eventArgument == this.ID + "_Bottom")
			{
				OnBottomClick(EventArgs.Empty);
			}
			else
			{
				base.RaisePostBackEvent(eventArgument);
			}
		}

		#endregion Methods

		#region Events

		/// <summary>
		/// Raises the BottomClick event.
		/// </summary>
		/// <param name="e">
		/// The event data.
		/// </param>
		protected virtual void OnBottomClick(EventArgs e)
		{
			EventHandler bottomClickHandler = this.Events[LargeSplitRibbonButton.BottomClickEvent] as EventHandler;

			if (bottomClickHandler != null)
			{
				bottomClickHandler(this, e);
			}
		}

		/// <summary>
		/// Occurs when the bottom part of the button is clicked.
		/// </summary>
		public event EventHandler BottomClick
		{
			add
			{
				this.Events.AddHandler(LargeSplitRibbonButton.BottomClickEvent, value);
			}
			remove
			{
				this.Events.RemoveHandler(LargeSplitRibbonButton.BottomClickEvent, value);
			}
		}

		#endregion Events

		#region Implementation Methods

		/// <summary>
		/// Creates the control's layout.
		/// </summary>
		protected override void CreateChildControls()
		{
			ClearControls();

			this.Attributes.Add("class", "largeSplitRibbonButton");
			this.Attributes.Add("title", this.ToolTip);

			HtmlGenericControl topDiv = new HtmlGenericControl("div");
			topDiv.EnableViewState = false;
			topDiv.ID = "Top";

			this.Controls.Add(topDiv);

			HtmlGenericControl topStartDiv = new HtmlGenericControl("div");
			topStartDiv.EnableViewState = false;
			topStartDiv.ID = "TopStart";
			topStartDiv.Attributes.Add("class", "largeSplitRibbonButtonTopStart");

			topDiv.Controls.Add(topStartDiv);

			HtmlGenericControl topItemDiv = new HtmlGenericControl("div");
			topItemDiv.EnableViewState = false;
			topItemDiv.ID = "TopItem";
			topItemDiv.Attributes.Add("class", "largeSplitRibbonButtonTopItem");

			topDiv.Controls.Add(topItemDiv);

			HtmlGenericControl imageDiv = new HtmlGenericControl("div");
			imageDiv.EnableViewState = false;
			imageDiv.Attributes.Add("class", "largeSplitRibbonButtonImage");

			topItemDiv.Controls.Add(imageDiv);

			HtmlImage image = new HtmlImage();
			image.EnableViewState = false;
			image.ID = "Image";

			imageDiv.Controls.Add(image);

			HtmlGenericControl topEndDiv = new HtmlGenericControl("div");
			topEndDiv.EnableViewState = false;
			topEndDiv.ID = "TopEnd";
			topEndDiv.Attributes.Add("class", "largeSplitRibbonButtonTopEnd");

			topDiv.Controls.Add(topEndDiv);

			HtmlGenericControl bottomDiv = new HtmlGenericControl("div");
			bottomDiv.EnableViewState = false;
			bottomDiv.ID = "Bottom";
			bottomDiv.Style.Add("clear", "both");

			this.Controls.Add(bottomDiv);

			HtmlGenericControl bottomStartDiv = new HtmlGenericControl("div");
			bottomStartDiv.EnableViewState = false;
			bottomStartDiv.ID = "BottomStart";
			bottomStartDiv.Attributes.Add("class", "largeSplitRibbonButtonBottomStart");

			bottomDiv.Controls.Add(bottomStartDiv);

			HtmlGenericControl bottomItemDiv = new HtmlGenericControl("div");
			bottomItemDiv.EnableViewState = false;
			bottomItemDiv.ID = "BottomItem";
			bottomItemDiv.Attributes.Add("class", "largeSplitRibbonButtonBottomItem");

			bottomDiv.Controls.Add(bottomItemDiv);

			HtmlGenericControl textDiv = new HtmlGenericControl("div");
			textDiv.EnableViewState = false;
			textDiv.ID = "Text";
			textDiv.Attributes.Add("class", "largeSplitRibbonButtonText");

			bottomItemDiv.Controls.Add(textDiv);

			HtmlGenericControl dropDownDiv = new HtmlGenericControl("div");
			dropDownDiv.EnableViewState = false;
			dropDownDiv.ID = "DropDownMenuImage";

			bottomItemDiv.Controls.Add(dropDownDiv);

			HtmlGenericControl bottomEndDiv = new HtmlGenericControl("div");
			bottomEndDiv.EnableViewState = false;
			bottomEndDiv.ID = "BottomEnd";
			bottomEndDiv.Attributes.Add("class", "largeSplitRibbonButtonBottomEnd");

			bottomDiv.Controls.Add(bottomEndDiv);
		}

		/// <summary>
		/// Create client script to cause a post back or execute client side java script.
		/// </summary>
		/// <param name="host">
		/// The host control to attach the onclick event to.
		/// </param>
		protected override void RenderClientPostBack()
		{
			string postBackReference = string.Empty;
			string bottomPostBackReference = string.Empty;
			string clientCode = string.Empty;
			string bottomClientCode = string.Empty;

			if (this.AutoPostBack)
			{
				if (this.Events[RibbonButton.ClickEvent] != null)
				{
					PostBackOptions options = new PostBackOptions(this);
					options.AutoPostBack = true;
					options.Argument = this.ID;

					postBackReference = this.Page.ClientScript.GetPostBackEventReference(options);
				}
				
				if (this.Events[LargeSplitRibbonButton.BottomClickEvent] != null)
				{
					PostBackOptions bottomOptions = new PostBackOptions(this);
					bottomOptions.AutoPostBack = true;
					bottomOptions.Argument = this.ID + "_Bottom";

					bottomPostBackReference = this.Page.ClientScript.GetPostBackEventReference(bottomOptions);
				}
			}

			if (!string.IsNullOrEmpty(this.OnClientClick))
			{
				clientCode = this.OnClientClick;
			}

			if (!string.IsNullOrEmpty(this.OnClientBottomClick))
			{
				bottomClientCode = this.OnClientBottomClick;
			}

			if ((!string.IsNullOrEmpty(clientCode)) || (!string.IsNullOrEmpty(postBackReference)))
			{
				HtmlGenericControl topDiv = FindControl("Top") as HtmlGenericControl;

				string code = "if (" + this.ClientMemberName + ".getEnabled()) {" + clientCode + postBackReference + "}";

				topDiv.Attributes.Add("onclick", code);
			}

			if ((!string.IsNullOrEmpty(bottomClientCode)) || (!string.IsNullOrEmpty(bottomPostBackReference)))
			{
				HtmlGenericControl bottomDiv = FindControl("Bottom") as HtmlGenericControl;

				string code = "if (" + this.ClientMemberName + ".getEnabled()) {" + 
					bottomClientCode + bottomPostBackReference + "}";

				bottomDiv.Attributes.Add("onclick", code);
			}
		}

		/// <summary>
		/// Sets the internal width of the button.
		/// </summary>
		protected override void SetWidth()
		{
			int innerWidth = 0;

			if (this.Width.Value > 4)
			{
				innerWidth = Convert.ToInt32(this.Width.Value) - 4;
			}

			HtmlGenericControl topItemDiv = FindControl("TopItem") as HtmlGenericControl;
			HtmlGenericControl bottomItemDiv = FindControl("BottomItem") as HtmlGenericControl;

			topItemDiv.Style.Add("width", innerWidth.ToString() + "px");
			bottomItemDiv.Style.Add("width", innerWidth.ToString() + "px");
		}

		/// <summary>
		/// Sets the button's checked state.
		/// </summary>
		protected override void SetCheckedState()
		{
			// Not appropriate for this control so ignore what base class would do
		}

		/// <summary>
		/// Sets whether the drop down image is visible.
		/// </summary>
		protected override void SetDropDownMenu()
		{
			base.SetDropDownMenu();

			// Now override the visible value and always set to true, as that is 
			// what this button type is intended for

			HtmlGenericControl dropDownDiv =
				FindControl("DropDownMenuImage") as HtmlGenericControl;

			if (dropDownDiv != null)
			{
				dropDownDiv.Visible = true;
			}
		}

		#endregion Implementation Methods

	}

}
