using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.ComponentModel;
using System.Web.UI.HtmlControls;

using RussellMason.Web.UI.Office2007.Design;

namespace RussellMason.Web.UI.Office2007
{

	/// <summary>
	/// Represents a Ribbon Group box for containing Ribbon Buttons.
	/// </summary>
	[
	Designer(typeof(RibbonGroupDesigner)),
	ToolboxData("<{0}:RibbonGroup runat=\"server\"></{0}:RibbonGroup>"),
	ParseChildren(true)
	]
	public class RibbonGroup : RibbonControl, IPostBackEventHandler
	{

		#region Fields

		private ITemplate _placeHolder = null;
		private ControlCollection _placeHolderControls = null;
		private bool _showDialogLauncher = false;
		private string _dialogLauncherScript = string.Empty;
		private string _dialogLauncherToolTip = string.Empty;
		private bool _autoPostBack = false;

		#endregion Fields

		#region Constructors
		
		/// <summary>
		/// Initializes a new instance of the RibbonGroup class.
		/// </summary>
		public RibbonGroup() : this(string.Empty)
		{
		} 

		/// <summary>
		/// Initializes a new instance of the RibbonGroup class.
		/// </summary>
		/// <param name="id">
		/// The ID to assign to the control.
		/// </param>
		public RibbonGroup(string id) : this(id, string.Empty)
		{
		}

		/// <summary>
		/// Initializes a new instance of the RibbonGroup class.
		/// </summary>
		/// <param name="id">
		/// The ID to assign to the control.
		/// </param>
		/// <param name="text">
		/// The group text.
		/// </param>
		public RibbonGroup(string id, string text)
		{
			// Shared State
			// 0 - Text
			this.SharedState.Add(new SharedStateItem("Text", typeof(string), string.Empty));

			this.ID = id;
			this.Text = text;
		} 

		#endregion Constructors

		#region Properties

		/// <summary>
		/// Gets or sets the content of the group.
		/// </summary>
		[
		DefaultValue(null),
		PersistenceMode(PersistenceMode.InnerProperty), 
		Browsable(false),
		TemplateInstance(TemplateInstance.Single)
		]
		public ITemplate PlaceHolder
		{
			get
			{
				return _placeHolder;
			}
			set
			{
				_placeHolder = value;
			}
		}

		/// <summary>
		/// Gets or sets the collection of controls to be added to the place holder at run time.
		/// Design-time controls are added first then run-time.
		/// </summary>
		[
		DefaultValue(null),
		Browsable(false)
		]
		public ControlCollection PlaceHolderControls
		{
			get
			{
				if (_placeHolderControls == null)
				{
					_placeHolderControls = new ControlCollection(this);
				}

				return _placeHolderControls;
			}
		}

		/// <summary>
		/// Gets or sets the text to display in the group title.
		/// </summary>
		[
		DefaultValue(""),
		Description("Specifies the text to display in the group title.")
		]
		public string Text
		{
			get
			{
				string text = Convert.ToString(this.SharedState["Text"].Value);

				return text;
			}
			set
			{
				this.SharedState["Text"].Value = value;
			}
		}

		/// <summary>
		/// Gets or sets whether the dialog launcher button is dislayed.
		/// </summary>
		/// </summary>
		[
		DefaultValue(false),
		Description("Specifies whether the dialog launcher button is dislayed.")
		]
		public bool ShowDialogLauncher
		{
			get
			{
				return _showDialogLauncher;
			}
			set
			{
				_showDialogLauncher = value;
			}
		}

		/// <summary>
		/// Gets or sets a tool tip to be displayed over the dialog launcher button.
		/// </summary>
		[
		DefaultValue(""),
		Description("Specifies the tool tip to display for the dialog launcher button.")
		]
		public string DialogLauncherToolTip
		{
			get
			{
				return _dialogLauncherToolTip;
			}
			set
			{
				_dialogLauncherToolTip = value;
			}
		}

		/// <summary>
		/// Gets or set the client code to execute when the dialog launcher button is clicked.
		/// </summary>
		[
		DefaultValue(""),
		Description("Specifies the client code to execute when the dialog laucher button is clicked.")
		]
		public virtual string OnClientDialogLauncherClick
		{
			get
			{
				return _dialogLauncherScript;
			}
			set
			{
				_dialogLauncherScript = value;
			}
		}

		/// <summary>
		/// Gets or sets whether clicking the dialog launcher button causes a post back.
		/// This must be explicitly set to cause a post back even if an event handler has 
		/// been defined.
		/// </summary>
		[
		DefaultValue(false),
		Description("Specifies whether clicking the dialog launcher button causes a post back.")
		]
		public bool AutoPostBack
		{
			get
			{
				return _autoPostBack;
			}
			set
			{
				_autoPostBack = value;
			}
		}

		#endregion Properties

		#region Methods

		/// <summary>
		/// Sets text - allows use by control and designer.
		/// </summary>
		public override void FinalizeRender()
		{
			base.FinalizeRender();

			SetWidth();
			SetText();
			SetDialogLaucherButtonState();

			if (!this.DesignMode)
			{
				RenderClientOnDialogLauncherClick();
			}
		}

		#endregion Methods

		#region Events

		/// <summary>
		/// Raises the DialogLauncherClick event.
		/// </summary>
		/// <param name="e">
		/// The event data.
		/// </param>
		protected virtual void OnDialogLauncherClick(EventArgs e)
		{
			if (this.DialogLauncherClick != null)
			{
				DialogLauncherClick(this, e);
			}
		}

		/// <summary>
		/// Occurs when the dialog launcher button is clicked.
		/// </summary>
		public event EventHandler DialogLauncherClick;

		#endregion Events

		#region Implementation Properties

		/// <summary>
		/// Gets the control's tag type.
		/// </summary>
		protected override HtmlTextWriterTag TagKey
		{
			get
			{
				return HtmlTextWriterTag.Div;
			}
		}

		#endregion Implementation Properties

		#region Implementation Methods

		/// <summary>
		/// Ensure child controls are available for referencing in the page load.
		/// </summary>
		/// <param name="e">
		/// The event data.
		/// </param>
		protected override void OnInit(EventArgs e)
		{
			base.OnInit(e);

			EnsureChildControls();
		}

		/// <summary>
		/// Ensure we add run-time controls so they are available for event handler re-attachment.
		/// </summary>
		/// <param name="e">
		/// The event data.
		/// </param>
		protected override void OnLoad(EventArgs e)
		{
			base.OnLoad(e);

			if (!this.DesignMode)
			{
				AddRuntimeControls();
			}
		}

		/// <summary>
		/// Creates the control's layout.
		/// </summary>
		protected override void CreateChildControls()
		{
			ClearControls();

			this.Attributes.Add("class", "ribbonGroupContainer ribbonPart");

			Table mainTable = new Table();
			mainTable.CellPadding = 0;
			mainTable.CellSpacing = 0;

			TableRow mainRow = new TableRow();
			mainTable.Rows.Add(mainRow);
			
			TableCell startCell = new TableCell();
			mainRow.Cells.Add(startCell);

			HtmlGenericControl startDiv = new HtmlGenericControl("div");
			startDiv.EnableViewState = false;
			startDiv.ID = "Start";
			startDiv.Attributes.Add("class", "ribbonGroupStart ribbonPart");

			startCell.Controls.Add(startDiv);

			TableCell middleCell = new TableCell();
			mainRow.Cells.Add(middleCell);

			HtmlGenericControl middleDiv = new HtmlGenericControl("div");
			middleDiv.EnableViewState = false;
			middleDiv.ID = "Item";
			middleDiv.Attributes.Add("class", "ribbonGroupItem ribbonPart");

			middleCell.Controls.Add(middleDiv);

			Table placeHolderTable = new Table();
			placeHolderTable.CellPadding = 0;
			placeHolderTable.CellSpacing = 0;

			TableRow placeHolderRow = new TableRow();
			placeHolderTable.Rows.Add(placeHolderRow);

			TableCell placeHolderCell = new TableCell();
			placeHolderCell.ID = "PlaceHolder";
			placeHolderCell.Attributes.Add("nowrap", "nowrap");
			placeHolderCell.CssClass = "ribbonGroupPlaceHolder ribbonPart";
			placeHolderCell.ColumnSpan = 2;

			placeHolderRow.Cells.Add(placeHolderCell);

			if (this.PlaceHolder != null)
			{
				Control control = new Control();
				this.PlaceHolder.InstantiateIn(control);
				placeHolderCell.Controls.Add(control);
			}

			TableRow textRow = new TableRow();
			placeHolderTable.Rows.Add(textRow);

			TableCell textCell = new TableCell();
			textRow.Cells.Add(textCell);

			HtmlGenericControl containerDiv = new HtmlGenericControl("div");
			containerDiv.Attributes.Add("class", "ribbonGroupTextContainer");
			textCell.Controls.Add(containerDiv);

			HtmlGenericControl dialogLauncherDiv = new HtmlGenericControl("div");
			dialogLauncherDiv.ID = "DialogLauncher";
			dialogLauncherDiv.Attributes.Add("class", "ribbonGroupDialogLauncher");

			containerDiv.Controls.Add(dialogLauncherDiv);

			HtmlGenericControl textDiv = new HtmlGenericControl("div");
			textDiv.ID = "Text";
			textDiv.Attributes.Add("class", "ribbonGroupText ribbonPart");

			containerDiv.Controls.Add(textDiv);

			middleDiv.Controls.Add(placeHolderTable);

			TableCell endCell = new TableCell();
			mainRow.Controls.Add(endCell);

			HtmlGenericControl endDiv = new HtmlGenericControl("div");
			endDiv.EnableViewState = false;
			endDiv.ID = "End";
			endDiv.Attributes.Add("class", "ribbonGroupEnd ribbonPart");

			endCell.Controls.Add(endDiv);

			this.Controls.Add(mainTable);
		}

		/// <summary>
		/// Create client script to cause a post back or execute client side java script.
		/// </summary>
		/// <param name="host">
		/// The host control to attach the onclick event to.
		/// </param>
		protected virtual void RenderClientOnDialogLauncherClick()
		{
			if (this.ShowDialogLauncher)
			{
				string postBackReference = string.Empty;
				string clientCode = string.Empty;

				HtmlGenericControl dialogLauncherDiv = FindControl("DialogLauncher") as HtmlGenericControl;

				if (this.AutoPostBack)
				{
					PostBackOptions options = new PostBackOptions(this);
					options.AutoPostBack = true;
					postBackReference = this.Page.ClientScript.GetPostBackEventReference(options);
				}

				if (!string.IsNullOrEmpty(this.OnClientDialogLauncherClick))
				{
					clientCode = this.OnClientDialogLauncherClick;
				}

				if ((!string.IsNullOrEmpty(clientCode)) || (!string.IsNullOrEmpty(postBackReference)))
				{
					string code = clientCode + postBackReference;

					dialogLauncherDiv.Attributes.Add("onclick", code);
				}
			}
		}

		/// <summary>
		/// Sets the internal width of the group.
		/// </summary>
		protected virtual void SetWidth()
		{
			if (!this.DesignMode)
			{
				this.Width = Unit.Empty;
			}

			if (!this.Width.IsEmpty)
			{
				int innerWidth = 0;

				if (this.Width.Value > 4)
				{
					innerWidth = Convert.ToInt32(this.Width.Value) - 5;
				}

				TableCell placeHolderCell = FindControl("PlaceHolder") as TableCell;

				placeHolderCell.Style.Add("width", innerWidth.ToString() + "px");
			}
		}

		/// <summary>
		/// Sets the group's text.
		/// </summary>
		protected virtual void SetText()
		{
			HtmlGenericControl textDiv = FindControl("Text") as HtmlGenericControl;

			if (textDiv != null)
			{
				textDiv.InnerText = this.Text;
			}
		}

		/// <summary>
		/// Shows or hides the dialog launcher button. 
		/// </summary>
		private void SetDialogLaucherButtonState()
		{
			//TableCell placeHolderCell = FindControl("DialogLauncherContainer") as TableCell;
			//placeHolderCell.Visible = this.ShowDialogLauncher;

			HtmlGenericControl dialogLauncherDiv = FindControl("DialogLauncher") as HtmlGenericControl;
			dialogLauncherDiv.Visible = this.ShowDialogLauncher;
			dialogLauncherDiv.Attributes.Add("title", this.DialogLauncherToolTip);
		} 

		/// <summary>
		/// Adds controls added at run-time before rendering.
		/// </summary>
		private void AddRuntimeControls()
		{
			TableCell placeHolderCell = FindControl("PlaceHolder") as TableCell;

			foreach (Control control in this.PlaceHolderControls)
			{
				placeHolderCell.Controls.Add(control);
			}
		}

		#endregion Implementation Methods

		#region IPostBackEventHandler Members

		/// <summary>
		/// Raises the click event;
		/// </summary>
		/// <param name="eventArgument">
		/// The event data.
		/// </param>
		public void RaisePostBackEvent(string eventArgument)
		{
			OnDialogLauncherClick(EventArgs.Empty);
		}

		#endregion IPostBackEventHandler Members

	}

}
