using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;

using RussellMason.Web.UI.Office2007.Design;

namespace RussellMason.Web.UI.Office2007
{

	/// <summary>
	/// Represents a small button for use on the Ribbon Bar which is made up of 2 parts,
	/// a button and a drop down list.
	/// </summary>
	[
	Designer(typeof(SmallRibbonButtonDesigner)),
	ToolboxData("<{0}:SmallSplitRibbonButton runat=\"server\"></{0}:SmallSplitRibbonButton>")
	]
	public class SmallSplitRibbonButton : SmallRibbonButton
	{

		#region Static Properties

		/// <summary>
		/// Gets the BottomClick event key.
		/// </summary>
		public static readonly object RightClickEvent = new object(); 

		#endregion Static Properties

		#region Fields

		private string _rightScript = string.Empty; 

		#endregion Fields

		#region Constructors

		/// <summary>
		/// Initializes a new instance of the SmallSplitRibbonButton class.
		/// </summary>
		public SmallSplitRibbonButton() : this(string.Empty)
		{
			// Shared State
			// 0 - Enabled
			// 1 - Checked
			// 2 - EnabledImageUrl
			// 3 - DisabledImageUrl
			// 4 - Text
			// 5 - DropDownMenuID
		}

		/// <summary>
		/// Initializes a new instance of the SmallSplitRibbonButton class.
		/// </summary>
		/// <param name="id">
		/// The ID to assign to the control.
		/// </param>
		public SmallSplitRibbonButton(string id) : this(id, string.Empty)
		{
		}

		/// <summary>
		/// Initializes a new instance of the SmallSplitRibbonButton class.
		/// </summary>
		/// <param name="id">
		/// The ID to assign to the control.
		/// </param>
		/// <param name="text">
		/// The button text.
		/// </param>
		public SmallSplitRibbonButton(string id, string text) : this(id, text, string.Empty, string.Empty)
		{
		}

		/// <summary>
		/// Initializes a new instance of the SmallSplitRibbonButton class.
		/// </summary>
		/// <param name="id">
		/// The ID to assign to the control.
		/// </param>
		/// <param name="text">
		/// The button text.
		/// </param>
		/// <param name="enabledImageUrl">
		/// The URL to an image to display when enabled.
		/// </param>
		/// <param name="disabledImageUrl">
		/// The URL to an image to display when disabled.
		/// </param>
		public SmallSplitRibbonButton(string id, string text, string enabledImageUrl, string disabledImageUrl)
		{
			this.ID = id;
			this.Text = text;
			this.EnabledImageUrl = enabledImageUrl;
			this.DisabledImageUrl = disabledImageUrl;

			// Default Width;
			this.Width = new Unit("45px");
		}

		#endregion Constructors

		#region Properties

		/// <summary>
		/// Gets or sets whether the button is in a checked state or not.
		/// This value cannot be changed and must be false. Setting to true
		/// causes an exception.
		/// </summary>
		[
		DefaultValue(false),
		Description("Specifies whether the button is in a checked state or not. Must be false for this control.")
		]
		public override bool Checked
		{
			get
			{
				bool isChecked = Convert.ToBoolean(this.SharedState["Checked"].Value);

				return isChecked;
			}
			set
			{
				if (value)
				{
					string message = "Checked cannot be set to true for this control.";

					throw new InvalidOperationException(message);
				}

				this.SharedState["Checked"].Value = value;
			}
		}

		/// <summary>
		/// Gets or set the client code to execute when the right part of the button is clicked.
		/// </summary>
		[
		DefaultValue(""),
		Description("Specifies the client code to execute when the right part of the button is clicked.")
		]
		public virtual string OnClientRightClick
		{
			get
			{
				return _rightScript;
			}
			set
			{
				_rightScript = value;
			}
		}

		#endregion Properties

		#region Methods

		/// <summary>
		/// Raises the click event;
		/// </summary>
		/// <param name="eventArgument">
		/// The event data.
		/// </param>
		public override void RaisePostBackEvent(string eventArgument)
		{
			if (eventArgument == this.ID + "_Right")
			{
				OnRightClick(EventArgs.Empty);
			}
			else
			{
				base.RaisePostBackEvent(eventArgument);
			}
		}

		#endregion Methods

		#region Events

		/// <summary>
		/// Raises the RightClick event.
		/// </summary>
		/// <param name="e">
		/// The event data.
		/// </param>
		protected virtual void OnRightClick(EventArgs e)
		{
			EventHandler rightClickHandler = this.Events[SmallSplitRibbonButton.RightClickEvent] as EventHandler;

			if (rightClickHandler != null)
			{
				rightClickHandler(this, e);
			}
		}

		/// <summary>
		/// Occurs when the right part of the button is clicked.
		/// </summary>
		public event EventHandler RightClick
		{
			add
			{
				this.Events.AddHandler(SmallSplitRibbonButton.RightClickEvent, value);
			}
			remove
			{
				this.Events.RemoveHandler(SmallSplitRibbonButton.RightClickEvent, value);
			}
		}

		#endregion Events

		#region Implementation Methods

		/// <summary>
		/// Creates the control's layout.
		/// </summary>
		protected override void CreateChildControls()
		{
			ClearControls();

			this.Attributes.Add("class", "smallSplitRibbonButton");
			this.Attributes.Add("title", this.ToolTip);

			HtmlGenericControl leftDiv = new HtmlGenericControl("div");
			leftDiv.EnableViewState = false;
			leftDiv.ID = "Left";
			leftDiv.Attributes.Add("class", "smallSplitRibbonButtonLeft");

			this.Controls.Add(leftDiv);

			HtmlGenericControl startDiv = new HtmlGenericControl("div");
			startDiv.EnableViewState = false;
			startDiv.ID = "LeftStart";
			startDiv.Attributes.Add("class", "smallSplitRibbonButtonLeftStart");

			leftDiv.Controls.Add(startDiv);

			HtmlGenericControl itemDiv = new HtmlGenericControl("div");
			itemDiv.EnableViewState = false;
			itemDiv.ID = "LeftItem";
			itemDiv.Attributes.Add("class", "smallSplitRibbonButtonLeftItem");

			leftDiv.Controls.Add(itemDiv);

			HtmlGenericControl imageDiv = new HtmlGenericControl("div");
			imageDiv.EnableViewState = false;
			imageDiv.Attributes.Add("class", "smallRibbonButtonImage");

			itemDiv.Controls.Add(imageDiv);

			HtmlImage image = new HtmlImage();
			image.EnableViewState = false;
			image.ID = "Image";

			imageDiv.Controls.Add(image);

			HtmlGenericControl textDiv = new HtmlGenericControl("div");
			textDiv.EnableViewState = false;
			textDiv.ID = "Text";
			textDiv.Attributes.Add("class", "smallRibbonButtonText");

			itemDiv.Controls.Add(textDiv);

			HtmlGenericControl endDiv = new HtmlGenericControl("div");
			endDiv.EnableViewState = false;
			endDiv.ID = "Right";
			endDiv.Attributes.Add("class", "smallSplitRibbonButtonRight");

			this.Controls.Add(endDiv);
		}

		/// <summary>
		/// Create client script to cause a post back or execute client side java script.
		/// </summary>
		/// <param name="host">
		/// The host control to attach the onclick event to.
		/// </param>
		protected override void RenderClientPostBack()
		{
			string postBackReference = string.Empty;
			string rightPostBackReference = string.Empty;
			string clientCode = string.Empty;
			string rightClientCode = string.Empty;

			if (this.AutoPostBack)
			{
				if (this.Events[RibbonButton.ClickEvent] != null)
				{
					PostBackOptions options = new PostBackOptions(this);
					options.AutoPostBack = true;
					options.Argument = this.ID;

					postBackReference = this.Page.ClientScript.GetPostBackEventReference(options);
				}

				if (this.Events[SmallSplitRibbonButton.RightClickEvent] != null)
				{
					PostBackOptions rightOptions = new PostBackOptions(this);
					rightOptions.AutoPostBack = true;
					rightOptions.Argument = this.ID + "_Right";

					rightPostBackReference = this.Page.ClientScript.GetPostBackEventReference(rightOptions);
				}
			}

			if (!string.IsNullOrEmpty(this.OnClientClick))
			{
				clientCode = this.OnClientClick;
			}

			if (!string.IsNullOrEmpty(this.OnClientRightClick))
			{
				rightClientCode = this.OnClientRightClick;
			}

			if ((!string.IsNullOrEmpty(clientCode)) || (!string.IsNullOrEmpty(postBackReference)))
			{
				HtmlGenericControl leftDiv = FindControl("Left") as HtmlGenericControl;

				string code = "if (" + this.ClientMemberName + ".getEnabled()) {" + clientCode + postBackReference + "}";

				leftDiv.Attributes.Add("onclick", code);
			}

			if ((!string.IsNullOrEmpty(rightClientCode)) || (!string.IsNullOrEmpty(rightPostBackReference)))
			{
				HtmlGenericControl rightDiv = FindControl("Right") as HtmlGenericControl;

				string code = "if (" + this.ClientMemberName + ".getEnabled()) {" +
					rightClientCode + rightPostBackReference + "}";

				rightDiv.Attributes.Add("onclick", code);
			}
		}

		/// <summary>
		/// Sets the internal width of the button.
		/// </summary>
		protected override void SetWidth()
		{
			int innerWidth = 0;

			if (this.Width.Value > 12)
			{
				innerWidth = Convert.ToInt32(this.Width.Value) - 15;  // LeftStart & Right
			}

			HtmlGenericControl leftDiv = FindControl("LeftItem") as HtmlGenericControl;

			leftDiv.Style.Add("width", innerWidth.ToString() + "px");
		}

		/// <summary>
		/// Sets the button's checked state.
		/// </summary>
		protected override void SetCheckedState()
		{
			// Not appropriate for this control so ignore what base class would do
		}

		/// <summary>
		/// Sets whether the drop down image is enabled.
		/// </summary>
		protected override void SetDropDownMenu()
		{
			HtmlGenericControl rightDiv = FindControl("Right") as HtmlGenericControl;

			if (rightDiv != null)
			{
				if (this.Enabled)
				{
					rightDiv.Attributes.Add("class", "smallSplitRibbonButtonRight");
				}
				else
				{
					rightDiv.Attributes.Add("class", "smallSplitRibbonButtonRightDisabled");
				}
			}
		}

		#endregion Implementation Methods

	}

}
